'***************************************************************************'
'* Filename : LED STRIP CLOCK                                              *'
'* Revision : 1.0                                                          *'
'* Controller : ATMEGA8535                                                 *'
'* Compiler: BASCOM-AVR 2.0.8.1                                            *'
'* Author: Rubashka Vasiliy , LNR , 2024                                   *'
'***************************************************************************'


'*                                      ATMEGA 8535
'*                                   --------  --------
'*                                  |        \/        |
'*      "4A"     o---------- PB0  -[| 1             40 |]- PA0 -----------o     "1A"
'*                                  |                  |
'*      "4B"     o---------- PB1  -[| 2             39 |]- PA1 -----------o     "1B"
'*                                  |                  |
'*      "4C"     o---------- PB2  -[| 3             38 |]- PA2 -----------o     "1C"
'*                                  |                  |
'*      "4D"     o---------- PB3  -[| 4             37 |]- PA3 -----------o     "1D"
'*                                  |                  |
'*      "4E"     o---------- PB4  -[| 5             36 |]- PA4 -----------o     "1E"
'*                                  |                  |
'*      "4F"     o---------- PB5  -[| 6             35 |]- PA5 -----------o     "1F"
'*                                  |                  |
'*      "4G"     o---------- PB6  -[| 7             34 |]- PA6 -----------o     "1G"
'*                                  |                  |
'*  "Hour +"     o---------- PB7  -[| 8             33 |]- PA7 -----------o     "Mine +"
'*                                  |                  |
'*      "+5V"    <-[10k]-- RESET  -[| 9             32 |]- AREF    -------
'*                                  |                  |          |       |
'*      "+5V"    <--*------- VCC  -[| 10            31 |]- GND  --*-||-  ===
'*                  |               |                  |          0,1uF|
'*                   -||-*-- GND  -[| 11            30 |]- AVCC -------*-->     "+5V"
'*                       |          |                  |
'*                -------  XTAL2  -[| 12            29 |]- PC7 -----------o     "SCL"
'*               |                  |                  |
'*              ===        XTAL1  -[| 13            28 |]- PC6 -----------o     "2G"
'*                                  |                  |
'*      "3A"     o---------- PD0  -[| 14            27 |]- PC5 -----------o     "2F"
'*                                  |                  |
'*      "3B"     o---------- PD1  -[| 15            26 |]- PC4 -----------o     "2E"
'*                                  |                  |
'*      "3C"     o---------- PD2  -[| 16            25 |]- PC3 -----------o     "2D"
'*                                  |                  |
'*      "3D"     o---------- PD3  -[| 17            24 |]- PC2 -----------o     "2C"
'*                                  |                  |
'*      "3E"     o---------- PD4  -[| 18            23 |]- PC1 -----------o     "2B"
'*                                  |                  |
'*      "3F"     o---------- PD5  -[| 19            22 |]- PC0 -----------o     "2A"
'*                                  |                  |
'*      "3G"     o---------- PD6  -[| 20            21 |]- PD7 -----------o     "SDA"
'*                                   ------------------

'PA -  
'PC -  
'PD -  
'PB -  

$regfile = "m8535.dat"                                      ' 
$crystal = 8000000                                          '   
$framesize = 128                                            ' ,   
$hwstack = 64                                               '  
$swstack = 64                                               '  

'=== 

'
Config Porta = Output
Config Portb = Output
Config Portc = Output
Config Portd = Output

'
Config Porta.7 = Input
Porta.7 = 1                                                 '   
Config Portb.7 = Input
Portb.7 = 1                                                 '   

                    '  RTC DS3231
'***************************************************************************************
' scl  sda 
Config Scl = Portc.7                                        'I2C Clock
Config Sda = Portd.7                                        'I2C Data
I2cinit

                    '    RTC DS3231 
'***************************************************************************************
Const Ds3231w = &HD0                                        '  
Const Ds3231r = &HD1                                        '-//- 
Const Control_reg = &H0E                                    '-//- 
Const Status_reg = &H0F                                     '-//- 
Const Stop_clock = &B10000000                               '  
Const Start_clock = &B00000000                              '  

                    '    
'***************************************************************************************
Dim Seco As Byte                                            '
Dim Mine As Byte                                            '
Dim Hour As Byte                                            '
Dim Ed_hour As Byte                                         ' 
Dim Des_hour As Byte                                        ' 
Dim Ed_min As Byte                                          ' 
Dim Des_min As Byte                                         ' 
Dim Temp As Byte                                            '



Gosub Gettime                                               ' 
'     -    
If Seco > 59 Or Mine > 59 Or Hour > 23 Then
   Seco = 0 : Mine = 0 : Hour = 0
   Gosub Settime
End If


'===================================..####.....###...===================================
'===================================..#..##...#..##..===================================
'===================================..#..##...#..##..===================================
'===================================..#..##...#..##..===================================
'===================================..####.....###...===================================
Do

   '  
   If Pinb.7 = 0 Then
      Incr Hour
      If Hour = 24 Then Hour = 0
      Gosub Settime                                         
      Waitms 400
   End If

   '  
   If Pina.7 = 0 Then
      Incr Mine
      If Mine = 60 Then Mine = 0
      Gosub Settime                                         
      Waitms 400
   End If

   '   DS3231
   Gosub Gettime

   '    
   If Seco.0 = 0 Then
      Admux = 0
   Else
      Admux = 64
   End If


   '   ULN2803
   Temp = Lookup(des_hour , Shrift)
   Porta.0 = Temp.0
   Porta.1 = Temp.1
   Porta.2 = Temp.2
   Porta.3 = Temp.3
   Porta.4 = Temp.4
   Porta.5 = Temp.5
   Porta.6 = Temp.6

   Temp = Lookup(ed_hour , Shrift)
   Portc.0 = Temp.0
   Portc.1 = Temp.1
   Portc.2 = Temp.2
   Portc.3 = Temp.3
   Portc.4 = Temp.4
   Portc.5 = Temp.5
   Portc.6 = Temp.6

   Temp = Lookup(des_min , Shrift)
   Portd.0 = Temp.0
   Portd.1 = Temp.1
   Portd.2 = Temp.2
   Portd.3 = Temp.3
   Portd.4 = Temp.4
   Portd.5 = Temp.5
   Portd.6 = Temp.6

   Temp = Lookup(ed_min , Shrift)
   Portb.0 = Temp.0
   Portb.1 = Temp.1
   Portb.2 = Temp.2
   Portb.3 = Temp.3
   Portb.4 = Temp.4
   Portb.5 = Temp.5
   Portb.6 = Temp.6


   Waitms 100
Loop
End
'===========================..#........###.....###....####...===========================
'===========================..#.......#..##...#..##...#..##..===========================
'===========================..#.......#..##...#..##...####...===========================
'===========================..#..##...#..##...#..##...#......===========================
'===========================..#####....###.....###....#......===========================
'=======================================================================================

'=======================================================================================
'===============================    ================================
'=======================================================================================


'==========    DS3231 ==========
'***************************************************************************************
Gettime:
   I2cstart
   I2cwbyte Ds3231w
   I2cwbyte &H00
   I2crepstart                                              '     I2c
   I2cwbyte Ds3231r
   I2crbyte Seco , Ack
   I2crbyte Mine , Ack
   I2crbyte Hour , Nack
   I2cstop
   '   - 
   Seco = Makedec(seco)
   Mine = Makedec(mine)
   Hour = Makedec(hour)
   '/ , 
   Des_hour = Hour / 10
   Ed_hour = Hour Mod 10
   Des_min = Mine / 10
   Ed_min = Mine Mod 10
Return


'==========    DS3231==========
'***************************************************************************************
Settime:
   Seco = Makebcd(seco)
   I2cstart
   I2cwbyte Ds3231w
   I2cwbyte 0
   I2cwbyte Seco
   I2cstop
   Mine = Makebcd(mine)
   I2cstart
   I2cwbyte Ds3231w
   I2cwbyte 1
   I2cwbyte Mine
   I2cstop
   Hour = Makebcd(hour)
   I2cstart
   I2cwbyte Ds3231w
   I2cwbyte 2
   I2cwbyte Hour
   I2cstop
Return


' 
'  
'
'   -- 1--
'  |      |
' 32      2
'  |      |
'   --64--
'  |      |
' 16      4
'  |      |
'   -- 8--
'          128
Shrift:                                                     '0-9
   Data 63 , 6 , 91 , 79 , 102 , 109 , 125 , 7 , 127 , 111